% ssp
% ssp-list(1)
% ssp 0.1.0
% 2024-01-09

# NAME

ssp-list - list entries

# SYNOPSIS

*ssp* list [`OPTION`..] [`PATTERN`]

# DESCRIPTION

The *list* command allows to list existing entries.

By default, all entries from the database are listed. If `PATTERN` is specified
however, only matching entries will be exported.

Pattern matching is performed by checking whether the entries' name match the
given `PATTERN`, which must be a shell wildcard pattern, as per [fnmatch](3).

This means you can use a question mark (`?`) to indicate one character exactly,
and a star (`*`) to indicate zero, one or more characters. Remember that those
shall most likely be escaped from your shell in order to be passed as-is to
*ssp*.

By default only the entries' names are listed (written in between double-quotes
and escaped as per the escaping rules for arguments described in [ssp](1)),
using a newline (`\n`) as separator.

You can use a different separator using *--sep*, or have the full entries' be
shown using *--details*.


# OPTIONS

: *-C*, *--no-comments*
:: Do not export entries' comments. Only applies with *--details* or *--format*.

: *-d*, *--details*
:: Show entries' details. Instead of only listing entries' names, show details
:: of each matching entries.
:: Note that by default secrets will /not/ be written out; specify *--secret*
:: in order to include them as well.

: *-e*, *--escape-comments*
:: Write comments within double-quotes (`"`) and escaped as per the escaping
:: rules described in [ssp](1) for arguments. Implies *--details*.

: *-f*, *--format*
:: Output a list in INI-like format. Refer to [ssp-show](1) for more on the
:: actual format.
::
:: Much like with *--details* this will show details of each matching entries,
:: but unlike *--details* secrets /will be/ included.
::
:: This is mainly intended for exporting data, and in fact the *export* command
:: is an alias for `list --format`

: *-S*, *--secret*
:: Show entries' secrets. Implies *--details*.

: *-s*, *--sep* `SEP`
:: Use `SEP` as separator instead of the default (a newline). This doesn't apply
:: with *--details* nor *--format*.
::
:: Note that entries' names are always written within double-quotes (and escaped
:: as per escaping rules described for arguments described in [ssp](1)).

# EXAMPLES

List all entries whose name contains "foo" - showing details and secrets but not
comments - from database `test.ssp` :

    $ ssp -D test.ssp l -SC '*foo*'


List all entries as a colon-separated list :

    $ ssp list -s:


# SEE ALSO

[ssp-export](1), [ssp-show](1)
