% ssp
% ssp-show(1)
% ssp 0.1.0
% 2024-01-09

# NAME

ssp-show - show an entry

# SYNOPSIS

*ssp* show OPTION[..] `ENTRY`

# DESCRIPTION

The *show* command allows to show an existing entry.

! HINT:
! You can only show one entry at a time, specifying its full name. It is
! possible to show multiple entries via pattern matching using the *list*
! command (and its *--details* option). Refer to [ssp-list](1) for more.

# OPTIONS

: *-C*, *--no-comments*
:: Do not export entries' comments.

: *-e*, *--escape-comments*
:: Write comments within double-quotes (`"`) and escaped as per the escaping
:: rules described in [ssp](1) for arguments.

: *-f*, *--format*
:: Output a list in INI-like format. See [[FORMAT]] below for more.
:: Implies *--secret*
::
:: This is mainly intended for exporting data, and is used by the *export*
:: command.

: *-S*, *--secret*
:: Show entries' secrets (as base32-encoded string).

# FORMAT

When using *--format* details of the entry will be shown in an INI-like format,
mainly intended for import/export.

! HINT: Tip
! You can list more than one entry at a time in the same format using the
! *--format* option of the *list* command and its pattern matching feature.
!
! In order to export all entries from the database, simply use the *export*
! command (which is an alias for `list --format`).

Therefore, first the name of the entry will be featured within square brackets,
denoting a section name. As such, no double-quotes will be used. However, the
escaping rules (as described in [ssp](1) for arguments) still apply.

Each setting will then be written on a single line, in a `name=value` form. Here
are the list of names/settings :

: *algo*
:: Hashing algorithm. Same value as for the *--algo* option. Refer to
:: [ssp-add](1) for more.

: *digits*
:: Number of digits.

: *counter*
:: Indicate this is a counter-based password. The value is the next value to be
:: used for said counter.

: *time*
:: Indicate this is a time-based password. The value is always 1.

: *precision*
:: The precision for time-based password.

: *secret*
:: The secret as a base32-encoded string.

: *comments*
:: Comments associated with the entry, within double-quotes and escaped as per
:: the escaping rules described for arguments in [ssp](1).

# EXAMPLE

Show entry "foobar" including its secret (but not its comments) :

    $ ssp show --no-comm -S foobar

# SEE ALSO

[ssp-list](1), [ssp-export](1)
