/* This file is part of ssp                             https://lila.oss/ssp
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#include <errno.h>
#include <limb/command.h>
#include <limb/exitcode.h>
#include <limb/loadopt.h>
#include <limb/output.h>
#include "ssp.h"

enum {
    OPT_NO_COMMENTS = 1 << 0,
};

struct export {
    const char *ptrn;
    int options;
};

enum {
    ARGID_PATTERN = OPTID_FIRST,
};

COMMAND(export, "Export entries", "[OPTION..] [<pattern>]",
" -C, --no-comments                     Do not export comments\n"
);

static int
parse_cmdline(int argc, const char *argv[], const char usage[], struct export *ctx)
{
    const struct option options[] = {
        OPTION_ARG_NONE('C', "no-comments",             0,          OPTID_SHORTOPT),
        LOADOPT_ARGUMENTS,
        ARGUMENT_OPT( "pattern",                        0,          ARGID_PATTERN),
        LOADOPT_DONE
    };
    struct loadopt lo = LOADOPT_ZERO;

    int c;
    while ((c = loadopt(NULL, argc, argv, options, 0, NULL, 0, &lo))) switch (c) {
        case 'C':
            ctx->options |= OPT_NO_COMMENTS;
            break;

        case ARGID_PATTERN:
            ctx->ptrn = LO_ARG(&lo);
            break;

        case -1:
            diecmdusage(EX_USAGE, usage, &command_export);
        default:
            die(EX_SOFTWARE, "unexpected return value ", PMINT(c), " from loadopt");
    };

    return LO_CUR(&lo);
}

int
export_main(int argc, const char *argv[], const char *env[], const char usage[], void *ssp_)
{
    struct ssp *ssp = ssp_;
    struct export ctx = { .ptrn = NULL, .options = 0 };

    (void) env;
    parse_cmdline(argc, argv, usage, &ctx);

    int C = !!(ctx.options & OPT_NO_COMMENTS);
    const char *av[] = { NULL, "--format", (C) ? "--no-comments" : ctx.ptrn, ctx.ptrn };
    return run_command("list", 2 + C + !!ctx.ptrn, av, env, ssp);
}
