/* This file is part of ssp                             https://lila.oss/ssp
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#include <errno.h>
#include <limb/command.h>
#include <limb/exitcode.h>
#include <limb/loadopt.h>
#include <limb/output.h>
#include "ssp.h"

#define OPT_SECRET  1 << 1  /* must be the same in list */

struct remove {
    struct ssp *ssp;
    size_t entoff;
};

enum {
    ARGID_ENTRY = OPTID_FIRST,
};

COMMAND(remove, "Remove an entry", "<entry>", NULL);

static int
parse_cmdline(int argc, const char *argv[], const char usage[], struct remove *ctx)
{
    const struct option options[] = {
        LOADOPT_ARGUMENTS,
        ARGUMENT_REQ( "entry",                          OPT_PATH,   ARGID_ENTRY),
        LOADOPT_DONE
    };
    struct loadopt lo = LOADOPT_ZERO;

    int c;
    while ((c = loadopt(&ctx->ssp->sa, argc, argv, options, 0, NULL, 0, &lo))) switch (c) {

        case ARGID_ENTRY:
            ctx->entoff = LO_OFF(&lo);
            break;

        case -1:
            diecmdusage(EX_USAGE, usage, &command_remove);
        default:
            die(EX_SOFTWARE, "unexpected return value ", PMINT(c), " from loadopt");
    };

    return LO_CUR(&lo);
}

int
remove_main(int argc, const char *argv[], const char *env[], const char usage[], void *ssp_)
{
    struct ssp *ssp = ssp_;
    struct remove ctx = { .ssp = ssp };

    (void) env;
    parse_cmdline(argc, argv, usage, &ctx);

    int r = open_db(ssp, 1);
    if (r < 0)
        diefu(exitcode_from_errno(errno), "open database");

    const char *entry = ssp->sa.s + ctx.entoff;

    r = db_find(entry, ssp);
    if (r < 0)
        diefu(EX_DATA_ERR, "find entry ", ESC, entry, ESC);
    if (!r)
        dief(EX_DATA_ERR, "no entry ", ESC, entry, ESC, " in database");

    dbg("showing entry");
    show_entry(OPT_SECRET, ssp);

    quiet("Removing entry ", ESC, entry, ESC, "...");
    rebuild_db(entry, NULL, NULL, ssp);

    return 0;
}
